@@ -2,7 +2,7 @@ import React, { FC } from 'react';
 import Navigation from './Navbar';
 import { Container } from 'reactstrap';
 
-import { Router, Redirect } from '@reach/router';
+import { BrowserRouter as Router, Redirect, Switch, Route } from 'react-router-dom';
 import useMedia from 'use-media';
 import {
   AlertsPage,
@@ -69,25 +69,45 @@ const App: FC<AppProps> = ({ consolesLink }) => {
     >
       <Theme />
       <PathPrefixContext.Provider value={basePath}>
-        <Navigation consolesLink={consolesLink} />
-        <Container fluid style={{ paddingTop: 70 }}>
-          <Router basepath={`${basePath}`}>
-            <Redirect from="/" to={`graph`} noThrow />
-            {/*
+        <Router basename={basePath}>
+          <Navigation consolesLink={consolesLink} />
+          <Container fluid style={{ paddingTop: 70 }}>
+            <Switch>
+              <Redirect exact from="/" to={`graph`} />
+              {/*
               NOTE: Any route added here needs to also be added to the list of
               React-handled router paths ("reactRouterPaths") in /web/web.go.
             */}
-            <PanelListPage path="/graph" />
-            <AlertsPage path="/alerts" />
-            <ConfigPage path="/config" />
-            <FlagsPage path="/flags" />
-            <RulesPage path="/rules" />
-            <ServiceDiscoveryPage path="/service-discovery" />
-            <StatusPage path="/status" />
-            <TSDBStatusPage path="/tsdb-status" />
-            <TargetsPage path="/targets" />
-          </Router>
-        </Container>
+              <Route path="/graph">
+                <PanelListPage />
+              </Route>
+              <Route path="/alerts">
+                <AlertsPage />
+              </Route>
+              <Route path="/config">
+                <ConfigPage />
+              </Route>
+              <Route path="/flags">
+                <FlagsPage />
+              </Route>
+              <Route path="/rules">
+                <RulesPage />
+              </Route>
+              <Route path="/service-discovery">
+                <ServiceDiscoveryPage />
+              </Route>
+              <Route path="/status">
+                <StatusPage />
+              </Route>
+              <Route path="/tsdb-status">
+                <TSDBStatusPage />
+              </Route>
+              <Route path="/targets">
+                <TargetsPage />
+              </Route>
+            </Switch>
+          </Container>
+        </Router>
       </PathPrefixContext.Provider>
     </ThemeContext.Provider>
   );
